if has('lua')
    " neocompleteの設定

    " 起動時にneocompleteを有効にする
    let g:neocomplete#enable_at_startup = 1

    " 補完時のsmartcaseを有効にする
    let g:neocomplete#enable_smart_case = 1
    " 補完時のあいまい検索を有効にする
    let g:neocomplete#enable_fuzzy_completion = 1

    " シンタックス補完の候補対象の最小文字数を設定する
    let g:neocomplete#sources#syntax#min_keyword_length = 3
    " バッファ補完や辞書補完の候補対象となる文字数を設定する
    let g:neocomplete#min_keyword_length = 3
    " キーワード補完を開始する入力文字数を設定する
    let g:neocomplete#auto_completion_start_length = 2
    " 手動補完を開始する入力文字数を設定する
    let g:neocomplete#manual_completion_start_length = 0

    " ファイルタイプごとの辞書ファイルを設定する
    let g:neocomplete#sources#dictionary#dictionaries = {
                \ 'default': '',
                \ }

    " 補完をするためのキーワードパターンを設定する
    if !exists('g:neocomplete#keyword_patterns')
        let g:neocomplete#keyword_patterns = {}
    endif
    let g:neocomplete#keyword_patterns._ = '\h\w*'

    " 補完した候補を補完前に戻す
    inoremap <expr><C-g> neocomplete#undo_completion()
    " 補完対象の共通文字列を補完する
    inoremap <expr><C-l> neocomplete#complete_common_string()

    " 補完を確定して改行する
    inoremap <silent> <CR> <C-r>=<SID>my_cr_function()<CR>
    function! s:my_cr_function()
        return neocomplete#smart_close_popup() . "\<CR>"
    endfunction

    " TAB、Shift + TABキーで補完候補を選択する
    inoremap <expr><TAB> pumvisible() ? "\<C-n>" :
                \ <SID>check_back_space() ? "\<TAB>" :
                \ neocomplete#start_manual_complete()
    function! s:check_back_space()
        let col = col('.') - 1
        return !col || getline('.')[col - 1]  =~ '\s'
    endfunction
    inoremap <expr><S-TAB> pumvisible() ? "\<C-p>" : "\<C-h>"

    " 補完のポップアップメニューを閉じてカーソル前の文字を削除
    inoremap <expr><C-h> neocomplete#smart_close_popup()."\<C-h>"
    inoremap <expr><BS> neocomplete#smart_close_popup()."\<C-h>"
    " 補完のポップアップメニューを閉じる
    inoremap <expr><C-y> neocomplete#close_popup()
    " 補完をキャンセル
    inoremap <expr><C-e> neocomplete#cancel_popup()
else
    " neocomplcacheの設定

    " 起動時にneocomplcacheを有効にする
    let g:neocomplcache_enable_at_startup = 1

    " 補完時のsmartcaseを有効にする
    let g:neocomplcache_enable_smart_case = 1
    " キャメルケースの区切り文字で補完できるようにする
    let g:neocomplcache_enable_camel_case_completion = 1
    " スネークケースの区切り文字で補完できるようにする
    let g:neocomplcache_enable_underbar_completion = 1

    " シンタックス補完の候補対象となる文字数を設定する
    let g:neocomplcache_min_syntax_length = 3
    " バッファ補完や辞書補完の候補対象となる文字数を設定する
    let g:neocomplcache_min_keyword_length = 3
    " キーワード補完を開始する入力文字数を設定する
    let g:neocomplcache_auto_completion_start_length = 2
    " 手動補完を開始する入力文字数を設定する
    let g:neocomplcache_manual_completion_start_length = 0

    " ファイルタイプごとの辞書ファイルを設定する
    let g:neocomplcache_dictionary_filetype_lists = {
                \     'default' : '',
                \ }

    " 補完をするためのキーワードパターンを設定する
    if !exists('g:neocomplcache_keyword_patterns')
        let g:neocomplcache_keyword_patterns = {}
    endif
    let g:neocomplcache_keyword_patterns.default = '\h\w*'

    " 補完した候補を補完前に戻す
    inoremap <expr><C-g> neocomplcache#undo_completion()
    " 補完対象の共通文字列を補完する
    inoremap <expr><C-l> neocomplcache#complete_common_string()

    " 補完を確定して改行する
    inoremap <silent> <CR> <C-r>=<SID>my_cr_function()<CR>
    function! s:my_cr_function()
      return neocomplcache#smart_close_popup() . "\<CR>"
    endfunction

    " TAB、Shift + TABキーで補完候補を選択する
    inoremap <expr><TAB> pumvisible() ? "\<C-n>" :
                \ <SID>check_back_space() ? "\<TAB>" :
                \ neocomplcache#start_manual_complete()
    function! s:check_back_space()
        let col = col('.') - 1
        return !col || getline('.')[col - 1]  =~ '\s'
    endfunction
    inoremap <expr><S-TAB> pumvisible() ? "\<C-p>" : "\<C-h>"

    " 補完のポップアップメニューを閉じてカーソル前の文字を削除
    inoremap <expr><C-h> neocomplcache#smart_close_popup()."\<C-h>"
    inoremap <expr><BS> neocomplcache#smart_close_popup()."\<C-h>"
    " 補完のポップアップメニューを閉じる
    inoremap <expr><C-y> neocomplcache#close_popup()
    " 補完をキャンセル
    inoremap <expr><C-e> neocomplcache#cancel_popup()
endif
